//***************** (c) 2010 BTC Korporacja ***********************************
//                     http://www kamami.com
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************

#include "HD44780.h"

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteNibble(u8 nibbleToWrite)
{
	GPIO_ResetBits(EN_GPIO, LCD_EN);

	GPIO_WriteBit(LCD_GPIO, LCD_D4, (BitAction)(nibbleToWrite & 0x01));
	GPIO_WriteBit(LCD_GPIO, LCD_D5, (BitAction)(nibbleToWrite & 0x02));
	GPIO_WriteBit(LCD_GPIO, LCD_D6, (BitAction)(nibbleToWrite & 0x04));
	GPIO_WriteBit(LCD_GPIO, LCD_D7, (BitAction)(nibbleToWrite & 0x08));

	Delay_us(5);

	GPIO_SetBits(EN_GPIO, LCD_EN);
	Delay_us(5);
	GPIO_ResetBits(EN_GPIO, LCD_EN);
	Delay_us(50);
	
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

#ifdef LCD_ENABLE_READ
u8 LCD_ReadNibble(void)
{
	u8 tmp = 0;
	GPIO_SetBits(EN_GPIO, LCD_EN);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D4) << 0);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D5) << 1);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D6) << 2);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D7) << 3);
	GPIO_ResetBits(EN_GPIO, LCD_EN);
	Delay_us(50);
	return tmp;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
u8 LCD_ReadStatus(void)
{
	u8 status = 0;
	
	GPIO_InitStructure.GPIO_Pin   =  LCD_D4 | LCD_D5 | LCD_D6 | LCD_D7;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_IPU;
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
	
	GPIO_SetBits(LCD_GPIO, LCD_RW);
	GPIO_ReetBits(LCD_GPIO, LCD_RS);
	
	status |= (LCD_ReadNibble() << 4);
	status |= LCD_ReadNibble();
	
	GPIO_InitStructure.GPIO_Pin   =  LCD_D4 | LCD_D5 | LCD_D6 | LCD_D7;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
	
	return status;
}
//-----------------------------------------------------------------------------
#endif

//-----------------------------------------------------------------------------
void LCD_WriteData(u8 dataToWrite)
{
#ifdef LCD_ENABLE_READ
	GPIO_ResetBits(LCD_GPIO, LCD_RW);
#endif
	GPIO_SetBits(RS_GPIO, LCD_RS);
	
	LCD_WriteNibble(dataToWrite >> 4);		//zapis starszej polowy
	LCD_WriteNibble(dataToWrite & 0x0F);	//zapis mlodszej polowy
	Delay_ms(10);

#ifdef LCD_ENABLE_READ
	while(LCD_ReadStatus() & 0x80);
#endif
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteCommand(u8 commandToWrite)
{
#ifdef LCD_ENABLE_READ
	GPIO_ResetBits(LCD_GPIO, LCD_RW);
#endif
	GPIO_ResetBits(RS_GPIO, LCD_RS);
	LCD_WriteNibble(commandToWrite >> 4);
	LCD_WriteNibble(commandToWrite & 0x0F);
	Delay_ms(10);
	
#ifdef LCD_ENABLE_READ
	while(LCD_ReadStatus() & 0x80);
#endif
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteText(char * text)
{
	while(*text)
	  LCD_WriteData(*text++);
}
//-----------------------------------------------------------------------------
void LCD_ClearLine(int line)
{
	LCD_WriteTextXY("                ", 0, line);
}

//-----------------------------------------------------------------------------
void LCD_GoTo(unsigned char x, unsigned char y)
{
	switch (y){
		case 0:
		LCD_WriteCommand(HD44780_DDRAM_SET | (x));
		break;
		case 1:
		LCD_WriteCommand(HD44780_DDRAM_SET | (x+64));
		break;
		case 2:
		LCD_WriteCommand(HD44780_DDRAM_SET | (x+12));
		break;
		case 3:
		LCD_WriteCommand(HD44780_DDRAM_SET | (x+76));
		break;
		default:
		LCD_WriteCommand(HD44780_DDRAM_SET | (x));
		break;
	}
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteTextXY(char * text, u8 x, u8 y)
{
	LCD_GoTo(x,y);
	while(*text)
	  LCD_WriteData(*text++);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteBinary(u32 var, u8 bitCount)
{
	s8 i;
	
	for(i = (bitCount - 1); i >= 0; i--)
	{
		LCD_WriteData((var & (1 << i))?'1':'0');
	}
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_ShiftLeft(void)
{
	LCD_WriteCommand(HD44780_DISPLAY_CURSOR_SHIFT | HD44780_SHIFT_LEFT | HD44780_SHIFT_DISPLAY);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_ShiftRight(void)
{
	LCD_WriteCommand(HD44780_DISPLAY_CURSOR_SHIFT | HD44780_SHIFT_RIGHT | HD44780_SHIFT_DISPLAY);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_Initialization(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

//	vu8 i = 0;
//	vu32 delayCnt = 0;
/*	
	GPIO_InitStructure.GPIO_Pin   =  BLR|BLG|BLB;
	GPIO_InitStructure.GPIO_Speed =  GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	GPIO_Init(BL_GPIO, &GPIO_InitStructure);
*/
	//enable GPIO clock
  RCC_APB2PeriphClockCmd(RCC_LCD_GPIO | RCC_LCD_RS_GPIO | RCC_LCD_EN_GPIO, ENABLE);

	GPIO_InitStructure.GPIO_Pin   =  LCD_D4|LCD_D5|LCD_D6|LCD_D7;
	GPIO_InitStructure.GPIO_Speed =  GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin   =  LCD_RS;
	GPIO_InitStructure.GPIO_Speed =  GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	GPIO_Init(RS_GPIO, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin   =  LCD_EN;
	GPIO_InitStructure.GPIO_Speed =  GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	GPIO_Init(EN_GPIO, &GPIO_InitStructure);

	GPIO_WriteBit(EN_GPIO, LCD_EN, Bit_RESET);
	GPIO_WriteBit(RS_GPIO, LCD_RS, Bit_RESET);
	
	Delay_ms(100);

	LCD_WriteNibble(0x03);            
	Delay_ms(5);
	LCD_WriteNibble(0x03);            
	Delay_us(150);
	LCD_WriteNibble(0x03);            
	//Delay_ms(5);
	LCD_WriteNibble(0x02);            
	

	//Delay_ms(30);
	LCD_WriteCommand(HD44780_FUNCTION_SET | 
	                 HD44780_FONT5x7 | 
	                 HD44780_TWO_LINE | 
	                 HD44780_4_BIT);
	
	LCD_WriteCommand(HD44780_CLEAR); 

	LCD_WriteCommand(HD44780_DISPLAY_ONOFF | 
	                 HD44780_DISPLAY_OFF); 
	
	LCD_WriteCommand(HD44780_ENTRY_MODE | 
	                 HD44780_EM_SHIFT_CURSOR | 
	                 HD44780_EM_INCREMENT);
	
	LCD_WriteCommand(HD44780_DISPLAY_ONOFF | 
	                 HD44780_DISPLAY_ON |
	                 HD44780_CURSOR_OFF | 
	                 HD44780_CURSOR_NOBLINK);
	Delay_ms(50);

}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
